/**************************************************************************\
 *
 *  This file is part of the Coin family of 3D visualization libraries.
 *  Copyright (C) 1998-2003 by Systems in Motion.  All rights reserved.
 *
 *  This library is free software; you can redistribute it and / or
 *  modify it under the terms of the GNU General Public License
 *  version 2 as published by the Free Software Foundation.  See the
 *  file LICENSE.GPL at the root directory of this source distribution
 *  for more details.
 *
 *  If you desire to use this library in software that is incompatible
 *  with the GNU GPL, and / or you would like to take advantage of the
 *  additional benefits with regard to our support services, please
 *  contact Systems in Motion about acquiring a Coin Professional
 *  Edition License.  See <URL:http://www.coin3d.org> for more
 *  information.
 *
 *  Systems in Motion, Abels gate 5, Teknobyen, 7030 Trondheim, NORWAY
 *  <URL:http://www.sim.no>, <mailto:support@sim.no>
 *
\**************************************************************************/

#ifndef SOXT_EXAMINERVIEWER_H
#define SOXT_EXAMINERVIEWER_H

#include <Inventor/SbLinear.h>
#include <Inventor/Xt/viewers/SoXtFullViewer.h>

class SoSeparator;
class SoSwitch;
class SoTranslation;
class SoScale;

class SoXtThumbWheel;

// *************************************************************************

class SOXT_DLL_API SoXtExaminerViewer : public SoXtFullViewer {
  SOXT_OBJECT_HEADER(SoXtExaminerViewer, SoXtFullViewer);

public:
  SoXtExaminerViewer(Widget parent = NULL,
                        const char * name = NULL,
                        SbBool embed = TRUE,
                        SoXtFullViewer::BuildFlag flag = BUILD_ALL,
                        SoXtViewer::Type type = BROWSER);
  ~SoXtExaminerViewer();

  void setAnimationEnabled(const SbBool enable);
  SbBool isAnimationEnabled(void) const;

  void stopAnimating(void);
  SbBool isAnimating(void) const;

  void setFeedbackVisibility(const SbBool enable);
  SbBool isFeedbackVisible(void) const;

  void setFeedbackSize(const int size);
  int getFeedbackSize(void) const;

  virtual void setViewing(SbBool enable);
  virtual void setCamera(SoCamera * camera);
  virtual void setCursorEnabled(SbBool enable);

protected:
  SoXtExaminerViewer(Widget parent,
                        const char * name,
                        SbBool embed,
                        SoXtFullViewer::BuildFlag flag,
                        SoXtViewer::Type type,
                        SbBool build);

  virtual void leftWheelMotion(float val);
  virtual void bottomWheelMotion(float val);
  virtual void rightWheelMotion(float val);

  virtual void createViewerButtons(Widget parent, SbPList * buttonlist);

  virtual const char * getDefaultWidgetName(void) const;
  virtual const char * getDefaultTitle(void) const;
  virtual const char * getDefaultIconTitle(void) const;

  virtual SbBool processSoEvent(const SoEvent * const event);
  virtual void setSeekMode(SbBool enable);
  virtual void actualRedraw(void);

  virtual void afterRealizeHook(void);

private:
  class SoXtExaminerViewerP * pimpl;

  friend class SoGuiExaminerViewerP;
  friend class SoXtExaminerViewerP;


//// FIXME!: get rid of toolkit-specific stuff. 20020109 mortene.

#ifdef __COIN_SOWIN__
protected:
  virtual LRESULT onCommand(HWND window, UINT message, WPARAM wparam, LPARAM lparam);
#endif // ! __COIN_SOWIN__
};

#endif // ! SOXT_EXAMINERVIEWER_H
